/* ----
 * ---- file   : dsp_c64.c
 * ---- author : Bastian Spiegel <bs@tkscript.de>
 * ---- legal  : (c) 2013 by Bastian Spiegel. 
 * ----          Distributed under terms of the GNU GENERAL PUBLIC LICENSE (GPL). See 
 * ----          http://www.gnu.org/licenses/licenses.html#GPL or COPYING for further information.
 * ----
 * ---- info   : C64 kernel module. This is part of the "c64_tools" package.
 * ----
 * ---- changed: 06Sep2013, 08Sep2013, 10Sep2013, 11Sep2013, 15Sep2013
 * ----
 * ----
 */

#define __NO_VERSION__

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/io.h>
#include <linux/interrupt.h>
#include <linux/delay.h>

#include "../include/types.h"
#include "../include/arch/dm3730/dm3730.h"

#include "kmod.h"
#include "reg.h"
#include "dsp_c64.h"


/*--------------------------------------------------------------------------- dsp_init() */
void dsp_init(void) {

   // Disable GPTIMER5 functional clock (EN_GPT5) (DSPBIOS timer)
   ////reg32_bit_clear(IVA2_PER_CM_FCLKEN, 6);
   
   // Disable GPTIMER5 interface clock (EN_GPT5)
   ////reg32_bit_clear(IVA2_PER_CM_ICLKEN, 6);
   
   // Disable automatic transition (clear bit 0, bits 1..31 are reserved)
   reg32_write(IVA2_CM_CLKSTCTRL, 0);
   
   // Disable IVA2_CLK (clear bit 0, bits 1..31 are reserved)
   reg32_write(IVA2_CM_FCLKEN, 0);
   
   // Reset video sequencer (RST3 bit) and DSP (RST1 bit) (do NOT reset MMU or the GPP OS will freeze!!)
   ////reg32_write(IVA2_PRM_RSTCTRL, 5);
   reg32_write(IVA2_PRM_RSTCTRL, 1 + 4);
   
   // Enable IVA2_CLK (set bit 1, bits 1..31 are reserved)
   reg32_write(IVA2_CM_FCLKEN, 1);
   
   {
      if(c64_debug_level >= 10)
      {
         printk(KERN_INFO "c64: disabling IVA2_MMU2..\n");
      }
      
      // Disable MMU
      reg32_bit_clear(IVA2_MMU2_CNTL, 1);
      
      if(c64_debug_level >= 10)
      {
         printk(KERN_INFO "c64: ..OK, IVA2_MMU2 is now disabled.\n");
      }
   }

   if(0)
   {
      sU32 val;
      val = reg32_read(0x48004D00);
      printk(KERN_INFO "val=0x%08x\n", val);
   }

   // Enable automatic transition (sets reserved bit 1 ??)
   reg32_write(IVA2_CM_CLKSTCTRL, 3);
}


/*--------------------------------------------------------------------------- dsp_exit() */
void dsp_exit(void) {
   // Disable automatic transition (clear bit 0, bits 1..31 are reserved)
   reg32_write(IVA2_CM_CLKSTCTRL, 0);
   
   // Disable IVA2_CLK (clear bit 0, bits 1..31 are reserved)
   reg32_write(IVA2_CM_FCLKEN, 0);

   // Enable automatic transition (sets reserved bit 1 ??)
   reg32_write(IVA2_CM_CLKSTCTRL, 3);
}
