/* ----
 * ---- file   : c64_dev.h
 * ---- author : Bastian Spiegel <bs@tkscript.de>
 * ---- legal  : (c) 2013 by Bastian Spiegel. 
 * ----          Distributed under terms of the GNU GENERAL PUBLIC LICENSE (GPL). See 
 * ----          http://www.gnu.org/licenses/licenses.html#GPL or COPYING for further information.
 * ----
 * ---- info   : Shared header file for c64 kernel module / userspace apps. This is part of the "c64_tools" package.
 * ----
 * ---- changed: 16Sep2013, 18Sep2013
 * ----
 * ----
 */

#ifndef __C64_TOOLS_C64_DEV_H__
#define __C64_TOOLS_C64_DEV_H__


#define C64_DEV_MAJOR  (239u)
#define C64_DEV_MINOR  ( 64u)
#define C64_DEV_NAME   "c64"


// for C64_IOCTL_SHM_ALLOC:
typedef struct {
   dsp_mem_region_t mem;  /* caller must set 'size' field, kmod will return virt_addr/phys_addr on success */
} c64_dev_ioctl_shm_alloc_t;


// for C64_IOCTL_CACHE:
#define C64_CACHE_AC_WB    (1u)  /* writeback */
#define C64_CACHE_AC_INV   (2u)  /* invalidate */
#define C64_CACHE_AC_WBINV (3u)  /* writeback and invalidate */

typedef struct {
   sU32 phys_addr;
   sU32 size;
   sU32 action;  /* one of C64_CACHE_AC_xxx */
} c64_dev_ioctl_cache_t;


// for C64_IOCTL_DEBUG_USR:
typedef union {
   sU32 idx;
   sU32 val;
} c64_dev_ioctl_debug_usr_t;


/* C64 IOCTL enumeration
 *
 *  (note) as a side note, why do the IOCTL numbers have to be unique system-wide?
 *          via udev we have dynamic device creation, there's dynamic device numbers, then this ? /rant
 */
#define C64_IOCTL_BASE C64_DEV_MAJOR

#define C64_IOCTL_SHM_ALLOC  _IOWR(C64_IOCTL_BASE, 0, c64_dev_ioctl_shm_alloc_t *)
#define C64_IOCTL_SHM_FREE   _IO  (C64_IOCTL_BASE, 1)
#define C64_IOCTL_TEST       _IO  (C64_IOCTL_BASE, 2)
#define C64_IOCTL_CACHE      _IOR (C64_IOCTL_BASE, 3, c64_dev_ioctl_cache_t *)
#define C64_IOCTL_DEBUG_USR  _IOWR(C64_IOCTL_BASE, 4, c64_dev_ioctl_debug_usr_t *)


#endif /* __C64_TOOLS_C64_DEV_H__ */
