/* ----
 * ---- file   : dsp_common.h
 * ---- author : Bastian Spiegel <bs@tkscript.de>
 * ---- legal  : (c) 2013 by Bastian Spiegel. 
 * ----          Distributed under terms of the LESSER GNU GENERAL PUBLIC LICENSE (LGPL). See 
 * ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING_LGPL for further information.
 * ----
 * ---- info   : Common DSP types and structs. This is part of the "c64_tools" package.
 * ----
 * ---- changed: 06Sep2013, 08Sep2013, 11Sep2013, 13Sep2013, 14Sep2013, 15Sep2013, 17Sep2013
 * ----
 * ----
 */

#ifndef __C64_TOOLS_DSP_COMMON_H__
#define __C64_TOOLS_DSP_COMMON_H__

#include "cplusplus_begin.h"


#define DSP_RESET_VECTOR_CODESIZE  (0x00000080u)  // size of RESET_VECTOR memory region used for boot code
                                                  //  (actually only 32 bytes are used)
#define DSP_RESET_VECTOR_MINSIZE   (0x00001000u)  // minimum size of RESET_VECTOR memory region

////#define DSP_RESET_VECTOR_MSGSIZE   (DSP_RESET_VECTOR_MINSIZE - DSP_RESET_VECTOR_CODESIZE)


#define DSP_PAGESIZE (4096u)  // (todo) use sysconf(_SC_PAGE_SIZE) [linux]

#define DSP_PAGESIZE_ROUND(a) ( ( ((a)+(DSP_PAGESIZE-1)) / DSP_PAGESIZE ) * DSP_PAGESIZE )

#define DSP_PAGESIZE_ALIGN(a) ( ( (a) / DSP_PAGESIZE ) * DSP_PAGESIZE )


/* for IPC / from GPP to DSP */
#if 0
#define DSP_MLB_RAM_MSGSERIAL       (dsp_config.reset_vector.phys_addr + DSP_RESET_VECTOR_CODESIZE +  0u)  // 0x86000080

/* (note) when the hack is enabled, this memory area simply stores the client handle (not read by the DSP, anyway) */
#define DSP_MLB_RAM_MSGTODSP        (dsp_config.reset_vector.phys_addr + DSP_RESET_VECTOR_CODESIZE + 16u)  // 0x86000090


/* for IPC / from DSP to GPP */
#define DSP_MLB_RAM_MSGTOGPP        (dsp_config.reset_vector.phys_addr + DSP_RESET_VECTOR_CODESIZE + 32u)  // 0x860000A0

#define DSP_MLB_RAM_WATCHDOG        (dsp_config.reset_vector.phys_addr + DSP_RESET_VECTOR_CODESIZE + 48u)  // 0x860000B0  (debugging)
#define DSP_MLB_RAM_DSPIRQCOUNT     (dsp_config.reset_vector.phys_addr + DSP_RESET_VECTOR_CODESIZE + 52u)  // 0x860000B4  (debugging)
#define DSP_MLB_RAM_DSPLASTREADMSG  (dsp_config.reset_vector.phys_addr + DSP_RESET_VECTOR_CODESIZE + 56u)  // 0x860000B8  (debugging)
#define DSP_MLB_RAM_DSPDEBUG        (dsp_config.reset_vector.phys_addr + DSP_RESET_VECTOR_CODESIZE + 60u)  // 0x860000BC

#else

#define DSP_IPC_BASE_GPP  (IVA2_L1DSRAM_BASE_GPP)
////(must use compiler generated array-access code or DSP will freeze) #define DSP_IPC_BASE_DSP  (IVA2_L1DSRAM_BASE_DSP)
#define DSP_IPC_SIZE  (64u)

// core IPC memory: (16 bytes)
#define DSP_IPC_MSG_SERIAL_OFF  ( 0u)
#define DSP_IPC_MSG_COMP_OFF    ( 4u)  // component id (lo) + cmd id (hi)
#define DSP_IPC_MSG_ARG1_OFF    ( 8u)
#define DSP_IPC_MSG_ARG2_OFF    (12u)
#define DSP_IPC_MSG_RET1_OFF    ( 8u)
#define DSP_IPC_MSG_RET2_OFF    (12u)

// component error number and internal debug memory:
#define DSP_IPC_ERRNO_OFF                 (16u)
#define DSP_IPC_DEBUG_DSPIRQCOUNT_OFF     (20u)
#define DSP_IPC_DEBUG_DSPITERCOUNT_OFF    (24u)
#define DSP_IPC_DEBUG_INTERNAL_OFF        (28u)

// user defined debug values:
#define DSP_IPC_DEBUG_USR0_OFF            (32u)
#define DSP_IPC_DEBUG_USR1_OFF            (36u)
#define DSP_IPC_DEBUG_USR2_OFF            (40u)
#define DSP_IPC_DEBUG_USR3_OFF            (44u)

// gpp side IPC addresses:
#define DSP_IPC_MSG_SERIAL_GPP  (DSP_IPC_BASE_GPP + DSP_IPC_MSG_SERIAL_OFF)
#define DSP_IPC_MSG_COMP_GPP    (DSP_IPC_BASE_GPP + DSP_IPC_MSG_COMP_OFF)
#define DSP_IPC_MSG_ARG1_GPP    (DSP_IPC_BASE_GPP + DSP_IPC_MSG_ARG1_OFF)
#define DSP_IPC_MSG_ARG2_GPP    (DSP_IPC_BASE_GPP + DSP_IPC_MSG_ARG2_OFF)
#define DSP_IPC_MSG_RET1_GPP    (DSP_IPC_BASE_GPP + DSP_IPC_MSG_RET1_OFF)
#define DSP_IPC_MSG_RET2_GPP    (DSP_IPC_BASE_GPP + DSP_IPC_MSG_RET2_OFF)

#define DSP_IPC_ERRNO_GPP                 (DSP_IPC_BASE_GPP + DSP_IPC_ERRNO_OFF)
#define DSP_IPC_DEBUG_DSPIRQCOUNT_GPP     (DSP_IPC_BASE_GPP + DSP_IPC_DEBUG_DSPIRQCOUNT_OFF)
#define DSP_IPC_DEBUG_DSPITERCOUNT_GPP    (DSP_IPC_BASE_GPP + DSP_IPC_DEBUG_DSPITERCOUNT_OFF)
#define DSP_IPC_DEBUG_INTERNAL_GPP        (DSP_IPC_BASE_GPP + DSP_IPC_DEBUG_INTERNAL_OFF)

#define DSP_IPC_DEBUG_USR0_GPP            (DSP_IPC_BASE_GPP + DSP_IPC_DEBUG_USR0_OFF)
#define DSP_IPC_DEBUG_USR1_GPP            (DSP_IPC_BASE_GPP + DSP_IPC_DEBUG_USR1_OFF)
#define DSP_IPC_DEBUG_USR2_GPP            (DSP_IPC_BASE_GPP + DSP_IPC_DEBUG_USR2_OFF)
#define DSP_IPC_DEBUG_USR3_GPP            (DSP_IPC_BASE_GPP + DSP_IPC_DEBUG_USR3_OFF)

#endif // 0


typedef struct {
   sU32 phys_addr;  // on DSP-side
   sU32 virt_addr;  // on ARM-side (mmap'd) (!= 0 while mapped)
   sU32 size;       // number of bytes
} dsp_mem_region_t;


#include "cplusplus_end.h"

#endif /* __C64_TOOLS_DSP_COMMON_H__ */
