/* ----
 * ---- file   : dsp_config.c
 * ---- author : Bastian Spiegel <bs@tkscript.de>
 * ---- legal  : (c) 2013 by Bastian Spiegel. 
 * ----          Distributed under terms of the LESSER GNU GENERAL PUBLIC LICENSE (LGPL). See 
 * ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING_LGPL for further information.
 * ----
 * ---- info   : DSP memory configuration. This is part of the "c64_tools" package.
 * ----
 * ---- changed: 06Sep2013, 07Sep2013, 08Sep2013, 11Sep2013, 13Sep2013, 18Sep2013
 * ----
 * ----
 */

#ifndef C64_KBUILD
#include "../../../include/types.h"
#include "../../../include/dsp_common.h"
#include "../../../include/dsp_config.h"
#endif // C64_KBUILD


#define CFG_PANDORA_128M     // 0x86000000..86400000 (boot linux w/ mem=96M@0x80000000) ("classic" edition)
//                           //  (can also be used as a shared config that works for both classic and rebirth/1ghz editions
//                                by setting up a memory hole in the linux bootargs, i.e.
//                                mem=96M@0x80000000 mem=412M@0x88000000 (rebirth/1Ghz) resp.
//                                mem=96M@0x80000000 mem=156M@0x88000000 (classic)
//                                )

//#define CFG_PANDORA_512M   // 0x9E000000..A0000000 (boot linux w/ mem=480m) ("rebirth" and "1ghz" editions)



#if defined(CFG_PANDORA_512M)
dsp_config_t dsp_config = {
#error This dsp_config causes kernel crashes although no syscall (mmap) reports any error
   .reset_vector = {
      0x9E000000u,
      0x00000000u,
      0x00001000u
   },
   
#ifndef CCS
   .ram = {
      0x9E001000u,
      0x00000000u,
      0x003FF000u
   },

#endif /* CCS */
};


#elif defined(CFG_PANDORA_128M)
dsp_config_t dsp_config = {
#ifndef CCS
   .reset_vector = {
      0x86000000u,
      0x00000000u,
      0x00001000u
   },
#else
   {
      0x86000000u,
      0x00000000u,
      0x00001000u
   },
#endif /* CCS */
   
#ifndef CCS
   .ram = {
      0x86001000u,
      0x00000000u,
      //0x00FCF000u
      0x003FF000u
   },

#endif /* CCS */
};

#else
#error No dsp_config defined. Please edit "dsp_config.c".
#endif
